'use strict';

var common = require('@lezer/common');
var lezerJulia = require('@plutojl/lezer-julia');
var language = require('@codemirror/language');
var autocomplete = require('@codemirror/autocomplete');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var autocomplete__namespace = /*#__PURE__*/_interopNamespaceDefault(autocomplete);

const juliaLanguage = language.LRLanguage.define({
    parser: lezerJulia.parser.configure({
        props: [
            language.indentNodeProp.add({
                IfStatement: language.continuedIndent({ except: /^\s*(end\b|else\b|elseif\b)/ }),
                TryStatement: language.continuedIndent({ except: /^\s*(end\b|else\b|finally\b)/ }),
                "Definition CompoundStatement": language.continuedIndent({ except: /^\s*(end\b)/ }), // node groups
                ExportStatement: language.continuedIndent(),
                ImportStatement: language.continuedIndent(),
                ReturnStatement: language.continuedIndent(),
                Assignment: language.continuedIndent(),
                BinaryExpression: language.continuedIndent(),
                TernaryExpression: language.continuedIndent(),
            }),
        ],
    }),
    languageData: {
        closeBrackets: { brackets: ["(", "[", "{", "'", '"', "`", "'''", '"""', "```"] },
        commentTokens: { line: "#", block: { open: "#=", close: "=#" } },
        indentOnInput: /^\s*(\]|\}|\)|end|else|elseif|catch|finally)$/,
    },
});
function collectKeywords() {
    let keywords = [];
    for (let node of lezerJulia.parser.nodeSet.types) {
        let groups = node.prop(common.NodeProp.group);
        let group = groups != null ? groups[0] : null;
        if (group === "keyword") {
            keywords.push({ label: node.name, type: "keyword" });
        }
    }
    return keywords;
}
const keywordCompletion = juliaLanguage.data.of({
    autocomplete: autocomplete__namespace.completeFromList(collectKeywords()),
});
const defaultConfig = {
    enableKeywordCompletion: false,
};
function julia(config = defaultConfig) {
    config = Object.assign(Object.assign({}, defaultConfig), config);
    let extensions = [];
    if (config.enableKeywordCompletion) {
        extensions.push(keywordCompletion);
    }
    return new language.LanguageSupport(juliaLanguage, extensions);
}

exports.julia = julia;
exports.juliaLanguage = juliaLanguage;
exports.keywordCompletion = keywordCompletion;
